@@ -100,13 +100,20 @@ static void initThemeHelper()
     // get config, with fallback to kdeglobals
     const auto config = KSharedConfig::openConfig();
 
-    // TODO: add Qt API to really fully override the engine
-    // https://codereview.qt-project.org/c/qt/qtbase/+/559108
-
     // enforce the theme configured by the user, with kdeglobals fallback
     // if not set, use Breeze
-    QString themeToUse = KConfigGroup(config, "Icons").readEntry("Theme", QStringLiteral("breeze"));
+    const QString themeToUse = KConfigGroup(config, "Icons").readEntry("Theme", QStringLiteral("breeze"));
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 8, 0)
+    // set our theme, Qt internally will still not fully use our engine and lookup
     QIcon::setThemeName(themeToUse);
+#else
+    // use Qt API to really fully override the engine, if we set KIconEngine the Key in our plugin will
+    // enforce that our engine is used
+    // https://codereview.qt-project.org/c/qt/qtbase/+/563241
+    QIcon::setThemeName(QStringLiteral("KIconEngine"));
+#endif
+
     // Tell KIconTheme about the theme, in case KIconLoader is used directly
     *_themeOverride() = themeToUse;
     qCDebug(KICONTHEMES) << "KIconTheme::initTheme() enforces the icon theme:" << themeToUse;
@@ -366,9 +373,6 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     // Find the theme description file. These are either locally in the :/icons resource path or global.
     QStringList icnlibs;
 
-    // local embedded icons have preference
-    icnlibs << QStringLiteral(":/icons");
-
 #ifdef Q_OS_ANDROID
     // Android icon theme installed by Kirigami
     icnlibs << QStringLiteral("assets:/qml/org/kde/kirigami");
@@ -380,6 +384,9 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     // These are not in the icon spec, but e.g. GNOME puts some icons there anyway.
     icnlibs += QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("pixmaps"), QStandardPaths::LocateDirectory);
 
+    // local embedded icons
+    icnlibs << QStringLiteral(":/icons");
+
     QString fileName;
     QString mainSection;
     const QString pathSuffix = QLatin1Char('/') + name + QLatin1Char('/');
@@ -544,6 +551,16 @@ static bool isAnyOrDirContext(const KIconThemeDir *dir, KIconLoader::Context con
     return context == KIconLoader::Any || context == dir->context();
 }
 
+QStringList KIconTheme::queryIcons() const
+{
+    QStringList result;
+    const auto listDirs = d->mDirs + d->mScaledDirs;
+    for (const auto &dir : listDirs) {
+        result.append(dir->iconList());
+    }
+    return result;
+}
+
 QStringList KIconTheme::queryIcons(int size, KIconLoader::Context context) const
 {
     // Try to find exact match
@@ -692,15 +709,15 @@ QStringList KIconTheme::list()
     // Find the theme description file. These are either locally in the :/icons resource path or global.
     QStringList icnlibs;
 
-    // local embedded icons have preference
-    icnlibs << QStringLiteral(":/icons");
-
     // global icons
     icnlibs += QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("icons"), QStandardPaths::LocateDirectory);
 
     // These are not in the icon spec, but e.g. GNOME puts some icons there anyway.
     icnlibs += QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("pixmaps"), QStandardPaths::LocateDirectory);
 
+    // local embedded icons
+    icnlibs << QStringLiteral(":/icons");
+
     for (const QString &iconDir : std::as_const(icnlibs)) {
         QDir dir(iconDir);
         const QStringList themeDirs = dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
